#! /usr/bin/env python3

import argparse
from collections import namedtuple
import ipaddress
import pathlib


def port_no(s):
    try:
        port = int(s)
        assert port > 0
        assert port < 65536
    except (ValueError, AssertionError):
        msg = '{!r} is not a valid port number.'.format(s)
        raise argparse.ArgumentTypeError(msg)
    return port


def positive_int(s):
    try:
        val = int(s)
        assert val > 0
    except (ValueError, AssertionError):
        msg = '{!r} is not a positive integer.'.format(s)
        raise argparse.ArgumentTypeError(msg)
    return val


class HostPortPair(namedtuple('_HostPortPair', 'ip port')):
    def __format__(self, spec):
        if self.ip.version == 6:
            return f'[{self.ip}]:{self.port}'
        return f'{self.ip}:{self.port}'

    def as_sockaddr(self):
        return str(self.ip), self.port


def host_port_pair(s):
    pieces = s.rsplit(':', maxsplit=1)
    if len(pieces) == 1:
        msg = f'{s!r} should contain both IP address and port number.'
        raise argparse.ArgumentTypeError(msg)
    elif len(pieces) == 2:
        try:
            # strip brackets in IPv6 hostname-port strings (RFC 3986).
            ip = ipaddress.ip_address(pieces[0].strip('[]'))
        except ValueError:
            msg = f'{pieces[0]!r} is not a valid IP address.'
            raise argparse.ArgumentTypeError(msg)
        try:
            port = int(pieces[1])
            assert port > 0
            assert port < 65536
        except (ValueError, AssertionError):
            msg = f'{pieces[1]!r} is not a valid port number.'
            raise argparse.ArgumentTypeError(msg)
    return HostPortPair(ip, port)


def ipaddr(s):
    try:
        ip = ipaddress.ip_address(s.strip('[]'))
    except ValueError as e:
        msg = f'{s!r} is not a valid IP address.'
        raise argparse.ArgumentTypeError(msg)
    return ip


def path(val):
    if val is None:
        return None
    p = pathlib.Path(val)
    if not p.exists():
        msg = f'{val!r} is not a valid file/dir path.'
        raise argparse.ArgumentTypeError(msg)
    return p



from setuptools import setup
from pathlib import Path

here = Path(__file__).resolve().parent


setup(
    name='sorna-common',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.9.0',
    description='Sorna common libraries',
    long_description='',
    url='https://github.com/lablup/sorna-common',
    author='Lablup Inc.',
    author_email='joongi@lablup.com',
    license='LGPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Environment :: No Input/Output (Daemon)',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
    ],

    packages=['sorna'],
    namespace_packages=['sorna'],

    python_requires='>=3.6',
    install_requires=[
        'simplejson',
        'pyzmq',
        'aiozmq',
        'aiohttp>=1.1',
        'async_timeout',
    ],
    extras_require={
        'dev': [],
        'test': ['pytest', 'pytest-asyncio', 'pytest-mock'],
    },
    data_files=[],

    entry_points={
        'console_scripts': [],
    },
)



#! /usr/bin/env python3

from collections import OrderedDict
from datetime import datetime
import logging
import urllib.request, urllib.error
import zmq

_logging_ctx = zmq.Context()


class LogstashHandler(logging.Handler):

    def __init__(self, endpoint):
        super().__init__()
        self.endpoint = endpoint
        self._cached_priv_ip = None
        self._cached_inst_id = None
        self._ctx = _logging_ctx
        self._sock = self._ctx.socket(zmq.PUB)
        self._sock.setsockopt(zmq.LINGER, 50)
        self._sock.setsockopt(zmq.SNDHWM, 20)
        self._sock.connect(self.endpoint)

    def emit(self, record):
        now = datetime.now().isoformat()
        tags = set()
        extra_data = dict()

        if record.exc_info:
            tags.add('has_exception')
            if self.formatter:
                extra_data['exception'] = self.formatter.formatException(record.exc_info)
            else:
                extra_data['exception'] = logging._defaultFormatter.formatException(record.exc_info)

        # This log format follows logstash's event format.
        log = OrderedDict([
            ('@timestamp', now),
            ('@version', 1),
            ('host', self._get_my_private_ip()),
            ('instance', self._get_my_instance_id()),
            ('logger', record.name),
            ('loc', '{0}:{1}:{2}'.format(record.pathname, record.funcName, record.lineno)),
            ('message', record.getMessage()),
            ('level', record.levelname),
            ('tags', list(tags)),
        ])
        log.update(extra_data)
        self._sock.send_json(log)

    def _get_my_private_ip(self):
        # See the full list of EC2 instance metadata at
        # http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html
        if self._cached_priv_ip is None:
            try:
                resp = urllib.request.urlopen('http://169.254.169.254/latest/meta-data/local-ipv4', timeout=0.2)
                self._cached_priv_ip = resp.read().decode('ascii')
            except urllib.error.URLError:
                self._cached_priv_ip = 'unavailable'
        return self._cached_priv_ip

    def _get_my_instance_id(self):
        if self._cached_inst_id is None:
            try:
                resp = urllib.request.urlopen('http://169.254.169.254/latest/meta-data/instance-id', timeout=0.2)
                self._cached_inst_id = resp.read().decode('ascii')
            except urllib.error.URLError:
                self._cached_inst_id = 'i-00000000'
        return self._cached_inst_id
